#!/usr/bin/env python

"""mite-cli

Usage:
    mite-cli new <project_dir> [options]

Options:
    --log-level=LEVEL       Set logger level: DEBUG, INFO, WARNING, ERROR, CRITICAL [default: INFO]
    -n --novenv             Don't create a python virtual environment
"""  # noqa: E501

import logging

from docopt import docopt

from .new_project import new_project

logger = logging.getLogger(__name__)


def setup_logging(opts):
    logging.basicConfig(
        level=opts["--log-level"],
        format="<%(levelname)s> %(message)s",
        force=True,
    )


def main():
    opts = docopt(__doc__)
    setup_logging(opts)
    if opts["new"]:
        new_project(opts["<project_dir>"], opts["--novenv"])


if __name__ == "__main__":
    main()



import logging
import os
import pathlib
from shutil import rmtree

from git import Repo
from yaspin import yaspin

from .envbuild_withreqs import EnvBuilderWithReqs

logger = logging.getLogger(__name__)


def new_project(project_dir: str, dont_create_venv: bool) -> None:
    if os.path.isdir(project_dir):
        logger.error("Directory already exists, please choose a new directory path")
        return False

    project_name = pathlib.Path(project_dir).name

    with yaspin(text="Cloning repo").dots10:
        Repo.clone_from("https://github.com/ryanlinnit-sky/mite-demo.git", project_dir)

    write_setup(os.path.join(project_dir, "setup.cfg"), project_name)

    rmtree(os.path.join(project_dir, ".git"))

    r = Repo.init(project_dir)
    r.index.add(["*", ".*"])
    r.index.commit("initial commit")

    if not dont_create_venv:
        new_venv(project_name, project_dir)


def write_setup(setup_cfg_filepath: str, project_name: str) -> None:
    with open(setup_cfg_filepath, "r") as fp:
        setup_cfg = fp.readlines()

    for index, line in enumerate(setup_cfg):
        if "mite-demo" in line:
            setup_cfg[index] = line.replace("mite-demo", project_name)

    with open(setup_cfg_filepath, "w") as fp:
        fp.writelines(setup_cfg)


def new_venv(project_name: str, project_dir: str):
    with yaspin(text="Creating virtual environment").dots10 as spinner:
        new_venv = EnvBuilderWithReqs(
            system_site_packages=False,
            clear=False,
            symlinks=False,
            upgrade=False,
            with_pip=True,
            prompt=None,
            project_dir=project_dir,
            spinner=spinner,
        )

        venv_dir = os.path.join(pathlib.Path.home(), ".virtualenvs", project_name)
        new_venv.create(venv_dir)

    logging.info(
        f"New virtual environment created. Activate with the command: source {venv_dir}/bin/activate"
    )



import logging
import shutil
import venv
from subprocess import PIPE, Popen
from threading import Thread

logger = logging.getLogger(__name__)


class EnvBuilderWithReqs(venv.EnvBuilder):
    def __init__(self, *args, **kwargs):
        self.project_dir = kwargs["project_dir"]
        self.spinner = kwargs["spinner"]
        del kwargs["project_dir"]
        del kwargs["spinner"]

        super().__init__(*args, **kwargs)

    def install_reader(self, stream):
        if self.spinner:
            self.spinner.text = "Installing packages"

        # Get the terminal width, so our output doesn't spill onto another line
        terminal_columns = shutil.get_terminal_size((80, 20)).columns - 25

        while True:
            s = stream.readline()
            if not s:
                break

            proc_output = s.decode("utf-8").strip()[:terminal_columns]
            if self.spinner:
                self.spinner.text = f"Installing packages [{proc_output}]"

        stream.close()

    def post_setup(self, context):
        pip_install_requirements = [
            context.env_exe,
            "-m",
            "pip",
            "install",
            "-e",
            ".",
        ]
        p = Popen(
            pip_install_requirements, stdout=PIPE, stderr=PIPE, cwd=self.project_dir
        )
        t1 = Thread(target=self.install_reader, args=(p.stdout,))
        t1.start()
        p.wait()
        t1.join()
        if p.returncode != 0:
            logger.error("Error when install python packages")
